
   
#define  SPI_ALLOC_STORAGE
#include "spi.h"

#include "gpio.h"
#include "nrf_log.h"
#include "nrf_gpiote.h"
#include "main.h"
#include "sc16is7x0.h"



static const nrf_drv_spi_t		SPI_left = NRF_DRV_SPI_INSTANCE(0);

static const nrf_drv_spi_t		SPI_right = NRF_DRV_SPI_INSTANCE(1);

static const nrf_drv_spi_t		SPI_flash = NRF_DRV_SPI_INSTANCE(2);



//*********************************************************************
// * @brief SPI user event handler.
 //* @param event
//*********************************************************************
void 
   spi_event_handler(
      nrf_drv_spi_evt_t const * 	p_event,
       void *                    	p_context)
{
   
   //NRF_LOG_INFO("line %d in spi_event_handler()", __LINE__ );
   
  
   if( p_event->type == NRF_DRV_SPI_EVENT_DONE )
   {
      // A SPI transfer finished but which one?
      if( p_context == (const nrf_drv_spi_t *)&SPI_left )
      {
	 // It was SPI_LEFT
	 
	 // Need to manage buffer
	 if( SPI_left_port_busy == SPI_TX_IN_PROGRESS )
	 {
	    // It was a transmit
	    SPI_left_tx_buff[ SPI_left_tx_deq_index ].busy = SPI_AVAILABLE;
	    SPI_left_tx_buff[ SPI_left_tx_deq_index ].bytes = 0;
	    
	    SPI_left_tx_deq_index++;
	    if( SPI_left_tx_deq_index >= SPI_TX_BUFFERS )
	    {
	       SPI_left_tx_deq_index = 0;
	    }
	 
	    // Check if there is anything else to be sent
	    if( SPI_left_tx_buff[ SPI_left_tx_deq_index ].busy == SPI_TX_IN_PROGRESS )
	    {
	       // Buffer waiting to be sent
	       nrf_drv_spi_transfer( &SPI_left,
				     SPI_left_tx_buff[ SPI_left_tx_deq_index ].data,	// tx ptr
				     SPI_left_tx_buff[ SPI_left_tx_deq_index ].bytes,	// tx len
				     NULL,		// rx ptr
				     0 );		// rx len
	    }
	    else
	    {
	       // Set hardware port to available
	       SPI_left_port_busy = SPI_AVAILABLE;
	    }
	 } // end of if( SPI_left_port_busy == SPI_TX_IN_PROGRESS )
	 else if( SPI_left_port_busy == SPI_RX_IN_PROGRESS )
	 {
	    // It was a receive
	    SPI_left_rx_buff[ SPI_left_rx_deq_index ].busy = SPI_RX_DONE;
	    
	    SPI_left_rx_deq_index++;
	    if( SPI_left_rx_deq_index >= SPI_RX_BUFFERS )
	    {
	       SPI_left_rx_deq_index = 0;
	    }
	 
	    // Check if there is anything else to be received
	    if( SPI_left_rx_buff[ SPI_left_rx_deq_index ].busy == SPI_RX_IN_PROGRESS )
	    {
	       // Buffer waiting to be received
	       nrf_drv_spi_transfer( &SPI_left,
				     NULL,
				     0,
				     SPI_left_rx_buff[ SPI_left_rx_deq_index ].data,	// rx ptr
				     SPI_left_rx_buff[ SPI_left_rx_deq_index ].bytes);	// rx len
	    }
	    else
	    {
	       // Set hardware port to available
	       SPI_left_port_busy = SPI_AVAILABLE;
	    }
	 } // end of if( SPI_left_port_busy == SPI_RX_IN_PROGRESS )
	 else
	 {
	    // Got a SPI Event Done indication but don't know why
	    NRF_LOG_DEBUG( "Bad state at line %d in %s", 
			   __LINE__,
			   __FILE__ );
	    SPI_left_port_busy = SPI_AVAILABLE;
	 }
      }
      else if( p_context == (const nrf_drv_spi_t *)&SPI_right )
      {
	 // It was SPI_LEFT
	 
	 // Need to manage buffer
	 if( SPI_left_port_busy == SPI_TX_IN_PROGRESS )
	 {
	    // It was a transmit
	    SPI_left_tx_buff[ SPI_left_tx_deq_index ].busy = SPI_AVAILABLE;
	    SPI_left_tx_buff[ SPI_left_tx_deq_index ].bytes = 0;
	    
	    SPI_left_tx_deq_index++;
	    if( SPI_left_tx_deq_index >= SPI_TX_BUFFERS )
	    {
	       SPI_left_tx_deq_index = 0;
	    }
	 
	    // Check if there is anything else to be sent
	    if( SPI_left_tx_buff[ SPI_left_tx_deq_index ].busy == SPI_TX_IN_PROGRESS )
	    {
	       // Buffer waiting to be sent
	       nrf_drv_spi_transfer( &SPI_left,
				     SPI_left_tx_buff[ SPI_left_tx_deq_index ].data,	// tx ptr
				     SPI_left_tx_buff[ SPI_left_tx_deq_index ].bytes,	// tx len
				     NULL,		// rx ptr
				     0 );		// rx len
	    }
	    else
	    {
	       // Set hardware port to available
	       SPI_left_port_busy = SPI_AVAILABLE;
	    }
	 } // end of if( SPI_left_port_busy == SPI_TX_IN_PROGRESS )
	 else if( SPI_left_port_busy == SPI_RX_IN_PROGRESS )
	 {
	    // It was a receive
	    SPI_left_rx_buff[ SPI_left_rx_deq_index ].busy = SPI_RX_DONE;
	    
	    SPI_left_rx_deq_index++;
	    if( SPI_left_rx_deq_index >= SPI_RX_BUFFERS )
	    {
	       SPI_left_rx_deq_index = 0;
	    }
	 
	    // Check if there is anything else to be received
	    if( SPI_left_rx_buff[ SPI_left_rx_deq_index ].busy == SPI_RX_IN_PROGRESS )
	    {
	       // Buffer waiting to be received
	       nrf_drv_spi_transfer( &SPI_left,
				     NULL,
				     0,
				     SPI_left_rx_buff[ SPI_left_rx_deq_index ].data,	// rx ptr
				     SPI_left_rx_buff[ SPI_left_rx_deq_index ].bytes);	// rx len
	    }
	    else
	    {
	       // Set hardware port to available
	       SPI_left_port_busy = SPI_AVAILABLE;
	    }
	 } // end of if( SPI_left_port_busy == SPI_RX_IN_PROGRESS )
	 else
	 {
	    // Got a SPI Event Done indication but don't know why
	    NRF_LOG_DEBUG( "Bad state at line %d in %s", 
			   __LINE__,
			   __FILE__ );
	    SPI_left_port_busy = SPI_AVAILABLE;
	 }	 
      } // end of if( p_context == (const nrf_drv_spi_t *)&SPI_right )
      else if( p_context == (const nrf_drv_spi_t *)&SPI_flash  )
      {
	 // It was SPI_FLASH
	 SPI_flash_busy = SPI_AVAILABLE;
      }
      else
      {
	 // It is a mystery
	 NRF_LOG_INFO( "Unexpected event at line %d in %s", __LINE__, __FILE__ );
      }
   }
   else
   {
      NRF_LOG_INFO( "p_event->type = %d", p_event->type );
   }
   
    return;
}  // end of spi_event_handler()


//*********************************************************************
//**@brief  Function for initializing the SPI module.
//
//**@snippet [SPI Initialization] */
//*********************************************************************
void 
   spi_init(
      void )
{
  
   nrf_drv_spi_config_t 	config = NRF_DRV_SPI_DEFAULT_CONFIG;
  
   uint32_t			ret_code;
   uint8_t			loop;

   
   // Clear the message buffer areas
   SPI_left_rx_enq_index = 0;
   SPI_right_rx_enq_index = 0;
   SPI_left_tx_enq_index = 0;
   SPI_right_tx_enq_index = 0;
   SPI_flash_enq_index = 0;
   
   SPI_left_rx_deq_index = 0;
   SPI_right_rx_deq_index = 0;
   SPI_left_tx_deq_index = 0;
   SPI_right_tx_deq_index = 0;
   SPI_flash_deq_index = 0;

   
   for( loop = 0; loop < SPI_TX_BUFFERS; loop++ )
   {
      SPI_left_tx_buff[loop].busy = SPI_AVAILABLE;
      SPI_left_tx_buff[loop].bytes = 0;
      memset( SPI_left_tx_buff[loop].data,
	      0,
	      SPI_TX_BUFFER_LEN );
      
      SPI_right_tx_buff[loop].busy = SPI_AVAILABLE;
      SPI_right_tx_buff[loop].bytes = 0;
      memset( SPI_right_tx_buff[loop].data,
	      0,
	      SPI_TX_BUFFER_LEN );
   }
   
   for( loop = 0; loop < SPI_RX_BUFFERS; loop++ )
   {
      SPI_left_rx_buff[loop].busy = SPI_AVAILABLE;
      SPI_left_rx_buff[loop].bytes = 0;
      memset( SPI_left_rx_buff[loop].data,
	      0,
	      SPI_RX_BUFFER_LEN );
      
      SPI_right_rx_buff[loop].busy = SPI_AVAILABLE;
      SPI_right_rx_buff[loop].bytes = 0;
      memset( SPI_right_rx_buff[loop].data,
	      0,
	      SPI_RX_BUFFER_LEN );      
   }
   
   for( loop = 0; loop < SPI_TX_BUFFERS; loop++ )
   {
      SPI_flash_buff[loop].busy = SPI_AVAILABLE;
      SPI_flash_buff[loop].bytes = 0;
      memset( SPI_flash_buff[loop].data,
	      0,
	      SPI_TX_BUFFER_LEN );
   }
   
   // Configure SPI0 (Left Port) first
   config.sck_pin =  	SPIM0_SCK_PIN;
   config.mosi_pin = 	SPIM0_MOSI_PIN;
   config.miso_pin = 	SPIM0_MISO_PIN;
   config.ss_pin = 	SPIM0_SS_PIN;	// active low
   config.irq_priority = NRFX_SPI_DEFAULT_CONFIG_IRQ_PRIORITY;
   config.orc = 	0xFF;
//   config.frequency = 	NRF_SPI_FREQ_4M;
//   config.mode = 	NRF_SPI_MODE_0;
//   config.bit_order = 	NRF_SPI_BIT_ORDER_MSB_FIRST;
     
   ret_code = nrf_drv_spi_init( &SPI_left, 
			        &config, 
			        spi_event_handler, 
				(void *) &SPI_left );	// context pointer
   
   if( ret_code != NRF_SUCCESS )
   {
      // Handle the error
      NRF_LOG_INFO( "nrf_drv_spi_init() failure at line %d", __LINE__ );
      app_error_fault_handler( 0, 0, 0);
   }
   else
   {
      // Configure the Left port spi-uart bridge
      NRF_LOG_INFO( "nrf_drv_spi_init() success at line %d", __LINE__ );
      spi_configure_chip( &SPI_left );
   }
   
   // Configure SPI1 (Right Port) next
   config.sck_pin =  	SPIM1_SCK_PIN;
   config.mosi_pin = 	SPIM1_MOSI_PIN;
   config.miso_pin = 	SPIM1_MISO_PIN;
   config.ss_pin = 	SPIM1_SS_PIN;	// active low
   config.irq_priority = NRFX_SPI_DEFAULT_CONFIG_IRQ_PRIORITY;
   config.orc = 	0xFF;
//   config.frequency = 	NRF_SPI_FREQ_4M;
//   config.mode = 	NRF_SPI_MODE_0;
//   config.bit_order = 	NRF_SPI_BIT_ORDER_MSB_FIRST;
     
   ret_code = nrf_drv_spi_init( &SPI_right, 
			       &config, 
			       spi_event_handler, 
			       (void *) &SPI_right );	// context pointer
   if( ret_code != NRF_SUCCESS )
   {
      // Handle the error
      NRF_LOG_INFO( "nrf_drv_spi_init() failure at line %d", __LINE__ );
      app_error_fault_handler( 0, 0, 0);
   }
   else
   {
      // Configure the Left port spi-uart bridge
      NRF_LOG_INFO( "nrf_drv_spi_init() success at line %d", __LINE__ );
      spi_configure_chip( &SPI_right );
   }

   // Configure SPI2 (SPI Flash) next
   config.sck_pin =  	SPIM2_SCK_PIN;
   config.mosi_pin = 	SPIM2_MOSI_PIN;
   config.miso_pin = 	SPIM2_MISO_PIN;
   config.ss_pin = 	SPIM2_SS_PIN;	// active low
   config.irq_priority = NRFX_SPI_DEFAULT_CONFIG_IRQ_PRIORITY;
   config.orc = 	0xFF;
//   config.frequency = 	NRF_SPI_FREQ_4M;
//   config.mode = 	NRF_SPI_MODE_0;
//   config.bit_order = 	NRF_SPI_BIT_ORDER_MSB_FIRST;
     
   ret_code = nrf_drv_spi_init( &SPI_flash,
			       &config, 
			       spi_event_handler,
			       (void *) &SPI_flash );
   
   if( ret_code != NRF_SUCCESS )
   {
      // Handle the error
      NRF_LOG_INFO( "nrf_drv_spi_init() failure at line %d", __LINE__ );
      app_error_fault_handler( 0, 0, 0);
   }


   return;

}  // end of spi_init()


//*********************************************************************
void 
   lp_irq_handler( 
      nrf_drv_gpiote_pin_t 			pin, 
      nrf_gpiote_polarity_t 			action)
{
   spi_interrupt_handler( pin, action );
   return;
}

//*********************************************************************
void 
   rp_irq_handler( 
      nrf_drv_gpiote_pin_t 			pin, 
      nrf_gpiote_polarity_t 			action)
{
   spi_interrupt_handler( pin, action );
   return;
}
//*********************************************************************
void
   spi_interrupt_handler(
      nrf_drv_gpiote_pin_t 			pin, 
      nrf_gpiote_polarity_t 			action)
{
   if( action == NRF_GPIOTE_POLARITY_LOTOHI )
   {
      NRF_LOG_INFO("pin = %d, action = LO to HI", pin );
   }
   else if( action == NRF_GPIOTE_POLARITY_HITOLO )
   {
      NRF_LOG_INFO("pin = %d, action = HI to LO", pin );
   }
   else if( action == NRF_GPIOTE_POLARITY_TOGGLE )
   {
      NRF_LOG_INFO("pin = %d, action = Toggle", pin );
   }
   else
   {
      NRF_LOG_INFO("pin = %d, action = unknown", pin );
   }
   
   return;			 
}
//*********************************************************************
void 
   spi_bootloader_enable( 
      sensor_head_id_t 			sensorHeadId ) 
{
 
   const nrf_drv_spi_t		*spi_instance;
   
   uint8_t			tx_buf[ 10 ];
   SPI_STATUS_E			*busy_flag;

   
   if( sensorHeadId == SENSOR_HEAD_1A_INDEX )
   {
      spi_instance = &SPI_left;
      busy_flag = &SPI_left_port_busy;

   }
   else if( sensorHeadId == SENSOR_HEAD_1B_INDEX )
   {
      spi_instance = &SPI_right;
      busy_flag = &SPI_right_port_busy;

   }
   else
   {
      spi_instance = NULL;
   }
   
   if( spi_instance != NULL )
   {
      // Set MCR to 0x00 to set RTS output hi (BOOT0=1, boot into bootloader)
      tx_buf[0] = 0x00 | (SC16IS7X0_MCR << SC16IS7X0_ADDR_SHIFT) ;	// R/W and address
      tx_buf[1] = 0x00;			// value
      *busy_flag = SPI_TX_IN_PROGRESS;
      
      APP_ERROR_CHECK(nrf_drv_spi_transfer( spi_instance,
                            tx_buf,
                            2,
                            NULL,
                            0 ) );
      while( *busy_flag != SPI_AVAILABLE )
      {
         delay_ms( 2 );
      } 
   }
   
   return;
} // end of spi_bootloader_enable()

//*********************************************************************
void 
   spi_bootloader_disable( 
      sensor_head_id_t 			sensorHeadId ) 
{
 
   const nrf_drv_spi_t		*spi_instance;
   
   uint8_t			tx_buf[ 10 ];
   SPI_STATUS_E			*busy_flag;

   
   if( sensorHeadId == SENSOR_HEAD_1A_INDEX )
   {
      spi_instance = &SPI_left;
      busy_flag = &SPI_left_port_busy;
   }
   else if( sensorHeadId == SENSOR_HEAD_1B_INDEX )
   {
      spi_instance = &SPI_right;
      busy_flag = &SPI_right_port_busy;
   }
   else
   {
      spi_instance = NULL;
   }
   
   if( spi_instance != NULL )
   {
      // Set MCR to 0x02 to set RTS output low (BOOT0=0, boot Flash)
      tx_buf[0] = 0x00 | (SC16IS7X0_MCR << SC16IS7X0_ADDR_SHIFT);	// R/W and address
      tx_buf[1] = 0x02;			// value
      *busy_flag = SPI_TX_IN_PROGRESS;
      
      nrf_drv_spi_transfer( spi_instance,
                            tx_buf,
                            2,
                            NULL,
                            0 );
      while( *busy_flag != SPI_AVAILABLE )
      {
         delay_ms( 2 );
      } 
   }
   
   return;
}  // end of spi_bootloader_disable()

//*********************************************************************
// spi_configure_chip()
//
// Configure baud rate generator to 16x480kbps based on 7.680MHz clock
// which requires a divide by 1.
//
// Enable RX FIFO
//
// Set RX FIFO interrupt level to 8 characters
//
// Enable RX timeout (4 character times since last received character)
//
//*********************************************************************
void
   spi_configure_chip(
      const nrf_drv_spi_t		*spi_instance )
      
{
   
   uint8_t			tx_buf[ 10 ];
   SPI_STATUS_E			*busy_flag;
   sensor_head_id_t		sensorHeadId;
   
   if( spi_instance != NULL )
   {
      if( spi_instance == &SPI_left )
      {
	 busy_flag = &SPI_left_port_busy;
         sensorHeadId = SENSOR_HEAD_1A_INDEX;
      }
      else
      {
	 busy_flag = &SPI_right_port_busy;
         sensorHeadId = SENSOR_HEAD_1B_INDEX;
      }
      
      // Set LCR to 0x80
      tx_buf[0] = SC16IS7X0_WRITE | (SC16IS7X0_LCR << SC16IS7X0_ADDR_SHIFT);	// R/W and address
      tx_buf[1] = 0x80;			// value
      *busy_flag = SPI_TX_IN_PROGRESS;
      
      nrf_drv_spi_transfer( spi_instance,
			    tx_buf,
			    2,
			    NULL,
			    0 );
      while( *busy_flag != SPI_AVAILABLE )
      {
	 delay_ms( 2 );
      }
      
      // Set DLL to 1
      tx_buf[0] = SC16IS7X0_WRITE | (SC16IS7X0_DLL << SC16IS7X0_ADDR_SHIFT);	// R/W and address
      tx_buf[1] = 0x1;			// value
      *busy_flag = SPI_TX_IN_PROGRESS;
      
      nrf_drv_spi_transfer( spi_instance,
			    tx_buf,
			    2,
			    NULL,
			    0 );
      while( *busy_flag != SPI_AVAILABLE )
      {
	 delay_ms( 2 );
      }
            
      // Set DHL to 0
      tx_buf[0] = SC16IS7X0_WRITE | (SC16IS7X0_DLH << SC16IS7X0_ADDR_SHIFT);	// R/W and address
      tx_buf[1] = 0x0;			// value
      *busy_flag = SPI_TX_IN_PROGRESS;
      
      nrf_drv_spi_transfer( spi_instance,
			    tx_buf,
			    2,
			    NULL,
			    0 );
      while( *busy_flag != SPI_AVAILABLE )
      {
	 delay_ms( 2 );
      }      
      
      // Set LCR to 0x03
      tx_buf[0] = SC16IS7X0_WRITE | ( SC16IS7X0_LCR << SC16IS7X0_ADDR_SHIFT);	// R/W and address
      tx_buf[1] = 0x03;			// value
      *busy_flag = SPI_TX_IN_PROGRESS;
      
      nrf_drv_spi_transfer( spi_instance,
			    tx_buf,
			    2,
			    NULL,
			    0 );
      while( *busy_flag != SPI_AVAILABLE )
      {
	 delay_ms( 2 );
      } 
   
      // Set MCR to 0x02 - Set RTS to low
      spi_bootloader_disable( sensorHeadId );
      
      // Setup FIFO
      //	- Set 0x01 to enable FIFOs
      //	- Set 0x02 to reset RX FIFO
      //	- Set 0x04 to reset TX FIFO
      //	- Set 0x00 to select 8 character level for RX FIFO interrupt
      // Set FCR to 0x07
      tx_buf[0] = SC16IS7X0_WRITE | (SC16IS7X0_FCR << SC16IS7X0_ADDR_SHIFT);	// R/W and address
      tx_buf[1] = 0x07;			// value
      *busy_flag = SPI_TX_IN_PROGRESS;
      
      nrf_drv_spi_transfer( spi_instance,
			    tx_buf,
			    2,
			    NULL,
			    0 );
      while( *busy_flag != SPI_AVAILABLE )
      {
	 delay_ms( 2 );
      } 
   
      // RX FIFO time-out does not require setup.
          
   }

   return;
} // end of spi_configure_chip()

//*********************************************************************
// spi_send()
//
// This routine sends data to a SPI connected device but requires
// access to local control structures.
//*********************************************************************
void
   spi_send(
      uint8_t		spi_num,
      uint8_t		*data_p,
      uint8_t		length )
 {
    
   const nrf_drv_spi_t		*spi_instance;
   SPI_STATUS_E			*busy_flag;
   uint8_t			len;
   int				msg_index;
   SPI_MSG_TX_BUF_T		*buff_p;
   
   
   if( spi_num == SPI_LEFT )
   {
      spi_instance = &SPI_left;
      busy_flag = &SPI_left_port_busy;
      msg_index = spi_get_tx_buff( spi_num );
      if( msg_index < 0 )
      {
	 spi_instance = NULL;
	 NRF_LOG_DEBUG( "no tx buff avail at line %d in %s", 
		        __LINE__,
			__FILE__ );
      }
      buff_p = &(SPI_left_tx_buff[msg_index]);
      NRF_LOG_INFO( "spi_send() for Left port" );
   }
   else if( spi_num == SPI_RIGHT )
   {
      spi_instance = &SPI_right;
      busy_flag = &SPI_right_port_busy;
      msg_index = spi_get_tx_buff( spi_num );
      if( msg_index < 0 )
      {
	 spi_instance = NULL;
	 NRF_LOG_DEBUG( "no tx buff avail at line %d in %s", 
		        __LINE__,
			__FILE__ );
      }
      buff_p = &(SPI_right_tx_buff[msg_index]);
      NRF_LOG_INFO( "spi_send() for Right port" );
   }
   else if( spi_num == SPI_FLASH )
   {
      spi_instance = &SPI_right;
      busy_flag = &SPI_flash_busy;
      msg_index = spi_get_tx_buff( spi_num );
      if( msg_index < 0 )
      {
	 spi_instance = NULL;
	 NRF_LOG_DEBUG( "no tx buff avail at line %d in %s", 
		        __LINE__,
			__FILE__ );
      }
      buff_p = &(SPI_flash_buff[msg_index]);
      NRF_LOG_INFO( "spi_send() for Flash" );
   }
   else
   {
      spi_instance = NULL;
      busy_flag = NULL;
   }
   
   if( spi_instance )
   {
      // No guarentee the passed in address is not on a stack
      if( length > SPI_TX_BUFFER_LEN )
      {
	 len = SPI_TX_BUFFER_LEN;
      }
      else
      {
	 len = length;
      }
      
      memcpy( ((uint8_t *)(buff_p->data)+1), data_p, len );
      
      buff_p->data[0] = SC16IS7X0_THR;	// set transmit holding register address
      len++;				// account for the THR address
      buff_p->bytes = len;
      buff_p->busy = SPI_TX_IN_PROGRESS;	// mark descriptor
      
      *busy_flag = SPI_TX_IN_PROGRESS;		// mark hardware
      nrf_drv_spi_transfer( spi_instance,
			    buff_p->data,	// tx ptr
			    len,		// tx len
			    NULL,		// rx ptr
			    0 );		// rx len
   }

    return;
 } // end of spi_send()

//*********************************************************************
// spi_get_tx_buff()
//
// Return the index of a buffer descriptor or -1 if no descriptors are
// available on the specified port
//*********************************************************************
int
   spi_get_tx_buff( uint8_t port )
{
   int		ret_index;
   
   if( port == SPI_LEFT )
   {
      if( SPI_left_tx_buff[SPI_left_tx_enq_index].busy != SPI_AVAILABLE )
      {
	 ret_index = -1;
      }
      else
      {
	 ret_index = ++SPI_left_tx_enq_index;
	 if( SPI_left_tx_enq_index > SPI_TX_BUFFERS )
	 {
	    SPI_left_tx_enq_index = 0;
	 }
      }
   }
   else if( port == SPI_RIGHT )
   {
      if( SPI_right_tx_buff[SPI_right_tx_enq_index].busy != SPI_AVAILABLE )
      {
	 ret_index = -1;
      }
      else
      {
	 ret_index = ++SPI_right_tx_enq_index;
	 if( SPI_right_tx_enq_index > SPI_TX_BUFFERS )
	 {
	    SPI_right_tx_enq_index = 0;
	 }
      }
   }
   else if( port == SPI_FLASH )
   {
      if( SPI_flash_buff[SPI_flash_enq_index].busy != SPI_AVAILABLE )
      {
	 ret_index = -1;
      }
      else
      {
	 ret_index = ++SPI_flash_enq_index;
	 if( SPI_flash_enq_index > SPI_TX_BUFFERS )
	 {
	    SPI_flash_enq_index = 0;
	 }
      }
   }
   else
   {
      ret_index = -1;
   }
   
   return( ret_index );
}  // end of spi_get_tx_buff()

//*********************************************************************
// spi_get_rx_buff()
//
// Return the index of a buffer descriptor or -1 if no descriptors are
// available on the specified port
//*********************************************************************
int
   spi_get_rx_buff( uint8_t port )
{
   int		ret_index;
   
   if( port == SPI_LEFT )
   {
      if( SPI_left_rx_buff[SPI_left_rx_enq_index].busy != SPI_AVAILABLE )
      {
	 ret_index = -1;
      }
      else
      {
	 ret_index = ++SPI_left_rx_enq_index;
	 if( SPI_left_rx_enq_index > SPI_RX_BUFFERS )
	 {
	    SPI_left_rx_enq_index = 0;
	 }
      }
   }
   else if( port == SPI_RIGHT )
   {
      if( SPI_right_rx_buff[SPI_right_rx_enq_index].busy != SPI_AVAILABLE )
      {
	 ret_index = -1;
      }
      else
      {
	 ret_index = ++SPI_right_rx_enq_index;
	 if( SPI_right_rx_enq_index > SPI_RX_BUFFERS )
	 {
	    SPI_right_rx_enq_index = 0;
	 }
      }
   }
   else if( port == SPI_FLASH )
   {
      if( SPI_flash_buff[SPI_flash_enq_index].busy != SPI_AVAILABLE )
      {
	 ret_index = -1;
      }
      else
      {
	 ret_index = ++SPI_flash_enq_index;
	 if( SPI_flash_enq_index > SPI_TX_BUFFERS )
	 {
	    SPI_flash_enq_index = 0;
	 }
      }
   }
   else
   {
      ret_index = -1;
   }
   
   return( ret_index );
}  // end of spi_get_rx_buff()

//*********************************************************************
// spi_free_tx_buff()
//
// Release a buffer from the specified port
//*********************************************************************
void
   spi_free_tx_buff(
      uint8_t		port,
      int		index )
{

   if( index < SPI_TX_BUFFERS )
   {
      if( port == SPI_LEFT )
      {
	 SPI_left_tx_buff[index].busy = SPI_AVAILABLE;
	 SPI_left_tx_buff[index].bytes = 0;
	 memset( SPI_left_tx_buff[index].data,
		0,
		SPI_TX_BUFFER_LEN );
      }
      else if( port == SPI_RIGHT )
      {
	 SPI_right_tx_buff[index].busy = SPI_AVAILABLE;
	 SPI_right_tx_buff[index].bytes = 0;
	 memset( SPI_right_tx_buff[index].data,
		0,
		SPI_TX_BUFFER_LEN );
      }
      else if( port == SPI_FLASH )
      {
	 SPI_flash_buff[index].busy = SPI_AVAILABLE;
	 SPI_flash_buff[index].bytes = 0;
	 memset( SPI_flash_buff[index].data,
		0,
		SPI_TX_BUFFER_LEN );
      }
      else
      {
	 NRF_LOG_DEBUG( "Wrong port at line %s in %d", __LINE__, __FILE__ );
      }
   }
   else
   {
      NRF_LOG_DEBUG( "index too large at line %d in %d", __LINE__, __FILE__ );
   }
   
   return;
}// spi_free_tx_buff()
	    
//*********************************************************************
// spi_free_rx_buff()
//
// Release a buffer from the specified port
//*********************************************************************
void
   spi_free_rx_buff(
      uint8_t		port,
      int		index )
{

   if( port == SPI_LEFT )
   {
      if( index < SPI_RX_BUFFERS )
      {
	 SPI_left_tx_buff[index].busy = SPI_AVAILABLE;
	 SPI_left_tx_buff[index].bytes = 0;
	 memset( SPI_left_tx_buff[index].data,
		0,
		SPI_TX_BUFFER_LEN );
      }
      else
      {
	 NRF_LOG_DEBUG( "index too large at line %d in %d", __LINE__, __FILE__ );
      }
   }
   else if( port == SPI_RIGHT )
   {
      if( index < SPI_RX_BUFFERS )
      {
	 
	 SPI_right_tx_buff[index].busy = SPI_AVAILABLE;
	 SPI_right_tx_buff[index].bytes = 0;
	 memset( SPI_right_tx_buff[index].data,
		0,
		SPI_TX_BUFFER_LEN );
      }
      else
      {
	 NRF_LOG_DEBUG( "index too large at line %d in %d", __LINE__, __FILE__ );
      }
   }
   else if( port == SPI_FLASH )
   {
      if( index < SPI_TX_BUFFERS )	// Flash uses TX size
      {
	 SPI_flash_buff[index].busy = SPI_AVAILABLE;
	 SPI_flash_buff[index].bytes = 0;
	 memset( SPI_flash_buff[index].data,
		0,
		SPI_TX_BUFFER_LEN );
      }
      else
      {
	 NRF_LOG_DEBUG( "index too large at line %d in %d", __LINE__, __FILE__ );
      }
   }
   else
   {
      NRF_LOG_DEBUG( "Wrong port at line %s in %d", __LINE__, __FILE__ );
   }
   
   return;
}// spi_free_rx_buff()
	    
//*********************************************************************
// spi_rx_msg_queued()
//
// Get a count of the number of spi rx messages queued
//*********************************************************************
uint8_t
   spi_rx_msg_queued(
      uint8_t			port )
{
   uint8_t		msg_count;
   
   msg_count = 0;
   
   
   
   
   return( msg_count );
} // end of spi_rx_msg_queued()
				     
   
//  end of file spi.c